/*
 * Unless otherwise specified through the '@author' tag or comments at
 * the top of the file or on a specific portion of the code the following license applies:
 *
 * Copyright (c) 2014, DoubleDoorDevelopment
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 *  Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 *  The header specified or the above copyright notice, this list of conditions
 *   and the following disclaimer below must be displayed at the top of the source code
 *   of any web page received while using any part of the service this software provides.
 *
 *   The header to be displayed:
 *       This page was generated by DoubleDoorDevelopment's D3Backend or a derivative thereof.
 *
 *  Neither the name of the project nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package net.doubledoordev.backend.web.socket;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import net.doubledoordev.backend.Main;
import net.doubledoordev.backend.permissions.User;
import net.doubledoordev.backend.server.JvmData;
import net.doubledoordev.backend.server.RestartingInfo;
import net.doubledoordev.backend.server.Server;
import net.doubledoordev.backend.util.Settings;
import net.doubledoordev.backend.util.TypeHellhole;
import net.doubledoordev.backend.util.WebSocketHelper;
import net.doubledoordev.backend.util.exceptions.AuthenticationException;
import org.glassfish.grizzly.websockets.DefaultWebSocket;
import org.glassfish.grizzly.websockets.WebSocket;
import org.glassfish.grizzly.websockets.WebSocketEngine;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import java.util.TimerTask;

import static net.doubledoordev.backend.util.Constants.*;

/**
 * @author Dries007
 */
public class AdvancedSettingsSocketApplication extends ServerWebSocketApplication
{
    public static final  HashMap<String, Data>             DATA_TYPES  = new HashMap<>();
    static
    {
        try
        {
            DATA_TYPES.put(RESTARTING_INFO, new Data(RestartingInfo.class, RESTARTING_INFO));
            DATA_TYPES.put(JVM_DATA, new Data(JvmData.class, JVM_DATA));
        }
        catch (NoSuchMethodException e)
        {
            throw new RuntimeException(e);
        }
    }
    private static final AdvancedSettingsSocketApplication APPLICATION = new AdvancedSettingsSocketApplication();
    private static final String                            URL_PATTERN = "/advancedsettings/*";

    private AdvancedSettingsSocketApplication()
    {
        TIMER.scheduleAtFixedRate(new TimerTask()
        {
            @Override
            public void run()
            {
                for (WebSocket socket : getWebSockets()) socket.sendPing("ping".getBytes());
            }
        }, SOCKET_PING_TIME, SOCKET_PING_TIME);
    }

    public static void register()
    {
        WebSocketEngine.getEngine().register(SOCKET_CONTEXT, URL_PATTERN, APPLICATION);
    }

    @Override
    public void onConnect(WebSocket socket)
    {
        super.onConnect(socket);

        Server server = (Server) ((DefaultWebSocket) socket).getUpgradeRequest().getAttribute(SERVER);
        if (server.isCoOwner((User) ((DefaultWebSocket) socket).getUpgradeRequest().getAttribute(USER)))
        {
            try
            {
                WebSocketHelper.sendData(socket, getData(server));
            }
            catch (Exception e)
            {
                WebSocketHelper.sendError(socket, e);
            }
        }
        else
        {
            WebSocketHelper.sendError(socket, new AuthenticationException());
            socket.close();
        }
    }

    @Override
    public void onMessage(WebSocket socket, String text)
    {
        Main.LOGGER.info(text);

        Server server = (Server) ((DefaultWebSocket) socket).getUpgradeRequest().getAttribute(SERVER);
        JsonObject object = JSONPARSER.parse(text).getAsJsonObject();
        for (String key : DATA_TYPES.keySet())
        {
            if (object.has(key))
            {
                try
                {
                    DATA_TYPES.get(key).setValues(server, object.getAsJsonObject(key));
                }
                catch (Exception e)
                {
                    WebSocketHelper.sendError(socket, e);
                }
            }
        }
        doSendUpdateToAll(server);
        Settings.save();
    }

    private void doSendUpdateToAll(Server server)
    {
        for (WebSocket socket : getWebSockets())
        {
            if (((DefaultWebSocket) socket).getUpgradeRequest().getAttribute(SERVER) != server) continue;
            if (!server.canUserControl((User) ((DefaultWebSocket) socket).getUpgradeRequest().getAttribute(USER))) continue;
            try
            {
                WebSocketHelper.sendData(socket, getData(server));
            }
            catch (Exception e)
            {
                e.printStackTrace();
                break;
            }
        }
    }

    public JsonObject getData(Server server) throws Exception
    {
        JsonObject object = new JsonObject();

        for (String key : DATA_TYPES.keySet())
        {
            object.add(key, GSON.toJsonTree(DATA_TYPES.get(key).getter.invoke(server)));
        }

        return object;
    }

    public static class Data
    {
        public final Class  clazz;
        public final Method getter;

        public Data(Class clazz, String getterName) throws NoSuchMethodException
        {
            this.clazz = clazz;
            Method m;
            try
            {
                m = Server.class.getDeclaredMethod(getterName);
            }
            catch (NoSuchMethodException ignored)
            {
                m = Server.class.getDeclaredMethod("get" + getterName);
            }
            this.getter = m;
        }

        public void setValues(Server server, JsonObject data) throws Exception
        {
            Object object = getter.invoke(server);
            for (Map.Entry<String, JsonElement> entry : data.entrySet())
            {
                TypeHellhole.set(clazz.getDeclaredField(entry.getKey()), object, entry.getValue());
            }
        }
    }
}
